#include <sys/time.h>
#include <time.h>
#include <stdio.h>
#include <stdlib.h>
#include <mpi.h>

int main(int argc, char**argv) {
  int comm_rank;
  int comm_size;

  int size=1024;
  MPI_Init(&argc, &argv);

  MPI_Comm_size(MPI_COMM_WORLD, &comm_size);
  MPI_Comm_rank(MPI_COMM_WORLD, &comm_rank);

  int *array = malloc(sizeof(int)*size);

  if(comm_rank==0) {
    srand(time(NULL));
    /* initialize the array */
    for(int i=0; i< size; i++) {
      array[i] = rand()% 1000000;
    }
  }

  /* broadcast the array */
  MPI_Bcast(array, size, MPI_INTEGER, 0, MPI_COMM_WORLD);

  int chunk_size=size/comm_size;
  int lower=chunk_size*comm_rank;
  int upper=chunk_size*(comm_rank+1);
  if(comm_rank==comm_size-1)
    upper = size;

  int min_val_local=array[lower];
  int max_val_local=array[lower];

  for(int i=lower; i<upper; i++) {
    if(array[i] < min_val_local)
      min_val_local = array[i];

    if(array[i] > max_val_local)
      max_val_local = array[i];
  }

  printf("I'm rank %d. Lower bound:%d. Upper bound: %d\n", comm_rank, lower, upper);

  int min_val;
  int max_val;
  MPI_Reduce(&min_val_local, &min_val, 1, MPI_INTEGER, MPI_MIN, 0, MPI_COMM_WORLD);
  MPI_Reduce(&max_val_local, &max_val, 1, MPI_INTEGER, MPI_MAX, 0, MPI_COMM_WORLD);

  if(comm_rank==0)
    printf("min: %d. max: %d\n", min_val, max_val);

  MPI_Finalize();
  return 0;
}
